/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2023
 */

import { FontAwesome } from "@expo/vector-icons"
import React, { useEffect } from "react"
import { useState } from "react"
import { Alert, Text, StyleSheet, TextInput, TouchableOpacity, View, Keyboard } from "react-native"
import Select from "react-native-multiple-select"
import { Landmark } from "../../../../data/landmarks"


interface ReportProps {
    landmark?: Landmark
    commentID?: string
    isLandmark: boolean
    reportLandmark?: (landmark, reasons, extraComments) => void
    reportComment?: (landmark, reasons, extraComments) => void
    toggleReporting: (state: boolean) => void
}

export const Report: React.FC<ReportProps> = (props) => {
    const [keyboardIsVisible, setKeyboardIsVisible] = useState(false)

    useEffect(() => {
        let isMounted = true;
        const showListener = Keyboard.addListener("keyboardDidShow", () => {
            if (isMounted) setKeyboardIsVisible(true)
        })
        const hideListener = Keyboard.addListener("keyboardDidHide", () => {
            if (isMounted) setKeyboardIsVisible(false)
        })

        return () => {
            showListener.remove()
            hideListener.remove()
            isMounted = false;
        }
    }, [])

    const onSelectedReasonsChange = (selectedReasons) => {
        setSelectedReasons(selectedReasons)
    }

    /**
     * Method that obtains the selected report reasons as strings and sends the appropriate report
     */
    const sendReport = () => {
        let reasonsToString = []
        for (let i = 0; i < selectedReasons.length; i++) {
            reasonsToString.push(reasons.find(item => item.value === selectedReasons[i]).label)
        }
        props.isLandmark ?
            props.reportLandmark(props.landmark, reasonsToString, text)
            : props.reportComment(props.commentID, reasonsToString, text)
        Alert.alert('Report submitted', 'Thank you for making The Atlas a safer place!')
        props.toggleReporting(false)
    }

    const [text, onChangeText] = React.useState('')
    const [selectedReasons, setSelectedReasons] = useState([])

    const reasons = props.isLandmark ?
        [
            { value: '1', label: 'Landmark does not exist' },
            { value: '2', label: 'Inaccurate tag' },
            { value: '3', label: 'Spam' },
            { value: '4', label: 'Sexually explicit material' },
            { value: '5', label: 'Hate speech or symbols' },
            { value: '6', label: 'Promotion of violence' },
        ]
        :
        [
            { value: '1', label: 'Misinformation' },
            { value: '2', label: 'Spam' },
            { value: '3', label: 'Sexually explicit material' },
            { value: '4', label: 'Hate speech or symbols' },
            { value: '5', label: 'Promotion of violence' },
            { value: '6', label: 'Harassment or bullying' },
            { value: '7', label: 'Impersonation' }
        ]

    return (
        <View>
            {props.isLandmark ?
                <Text style={{ color: 'white', marginBottom: 20, fontSize: 15 }}>Report Landmark</Text>
                :
                <Text style={{ color: 'white', marginBottom: 20, fontSize: 15 }}>Report Comment</Text>
            }
            <View style={{ width: '100%' }}>
                <View style={{ justifyContent: 'flex-start' }}>
                    <Text style={{ marginRight: 10, marginBottom: 5, color: 'white' }}>Reason(s) for reporting:</Text>
                    <View style={{ alignSelf: 'stretch', borderColor: "blue", borderWidth: 0, width: '100%', justifyContent: 'center' }}>
                        <Select  // Dropdown list to select any number of reasons for the report
                            styleRowList={{ borderColor: "red", borderWidth: 0 }}
                            styleIndicator={{ backgroundColor: "white" }}
                            textColor='black'
                            itemTextColor='black'
                            items={reasons}
                            displayKey="label"
                            uniqueKey="value"
                            submitButtonText="Confirm"
                            submitButtonColor='black'
                            onSelectedItemsChange={onSelectedReasonsChange}
                            selectedItems={selectedReasons}
                            textInputProps={{ editable: false, autoFocus: false }}
                            searchIcon={false}
                            searchInputPlaceholderText=''
                            tagBorderColor='white'
                            tagTextColor='white'
                            tagRemoveIconColor='white'
                            selectedItemTextColor='red'
                            selectedItemIconColor='red'>
                        </Select>
                    </View>
                </View>
                <Text style={{ marginRight: 10, marginBottom: 5, marginTop: 15, color: 'white' }}>
                    Optional extra comments: <FontAwesome size={16} name="question-circle-o" onPress={() => Alert.alert("", "This is space to provide any additional info about the report that you feel may not be captured by the reason(s) for reporting")} />      
                </Text>
                <TextInput  // Text box if the user wants to add extra comments to the report
                    style={styles.input}
                    onChangeText={onChangeText}
                    value={text}
                    placeholder="Optional extra comments..."
                    placeholderTextColor='gray'
                    keyboardType="default"
                />
            </View>
            {!keyboardIsVisible &&
                <View style={{ justifyContent: 'flex-end', flexDirection: 'row', paddingHorizontal: 20, paddingBottom: 10, marginTop: 5 }}>
                    <TouchableOpacity onPress={() => props.toggleReporting(false)}><Text style={{ color: 'white', marginRight: 25 }}>Cancel</Text></TouchableOpacity>
                    <TouchableOpacity
                        onPress={() =>
                            Alert.alert(
                                'Are you sure you want to submit this report?',
                                'This action cannot be undone.',
                                [
                                    { text: 'Cancel', onPress: () => console.log('Report cancelled'), style: 'cancel' },
                                    { text: 'OK', onPress: sendReport },
                                ],
                                { cancelable: false }
                            )}>
                        <Text style={{ color: 'white' }}>Submit Report</Text>
                    </TouchableOpacity>
                </View>
            }
        </View>
    )
}

const styles = StyleSheet.create({
    input: {
        height: 40,
        marginBottom: 20,
        borderWidth: 1,
        borderRadius: 0,
        padding: 10,
        borderColor: 'white',
        backgroundColor: 'white'

    },
})